==> caiviar-0.3.3.tar.gz: caiviar-0.3.3/src/CapiHandler2.h <==
/* CapiHandler2.h
 *
header for CapiHandler2.cpp

Part of the caiviar package. 

Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#define MAXCHANNELS 128

#ifndef __capihandler2_h__
#define __capihandler2_h__

#define MSGBUFSIZE 64

#include "../config.h"

#if defined(LINUX) || defined(CYGWIN)
# include <stdio.h>
# include <stdlib.h>
# include "Log.h"

#include "capi20.h"

# include "SimpleCapi.h"
#elif defined(WIN32)

# if _MSC_VER > 1000
#  pragma once
# endif

#include <winsock2.h>
#include <stdlib.h>
#include <malloc.h>

#include "Log.h"

#include "capi20.h"

#include <windef.h>
#include <winbase.h>
#include "SimpleCapi.h"
#endif

#include "os.h"

/* these are the states of some finite state automon.
 * The comments are the other states that can be reached from the
 * actual state. The functions in parentheses are the ones which
 * can be called for the state transition in question
 */
enum state_t
{
    state_idle,  // alertreq, alerted(DONT_DO_ALERT=1), calling(call())
    state_calling,  // connected, idle
    state_alertreq, // alerted
    state_alerted, // connected(answer()), idle(answer() failed)
    state_connected, // disconnecting(hangup from other end), idle(hangup())
    state_disconnecting, // idle
    state_holdreq,
    state_retrievereq,
    state_3ptyreq
};


/* at the moment, this job structure is only used for play and
 * record jobs. call is also a job, but doesn't have a structure.
 * The job id for calls is always 0x0300+line
 */
struct job {
    char * data;
    int data_size;
    char * data_pos;
    int bytes_remaining;
    short unsigned int datahandle;
    int jobid;
    void(*callback)(void*,int);
    void*callback_data;
} ;

class Queue
{
    char* queue;
    int readpos;
    int writepos;
    int number;
    int size;
    int nmemb;
    Mutex mutex;
    public:
    Queue(int size, int nmemb);
    ~Queue();
    int put(void*data);
    int get(void*data);
    int flush();
    int num();
};

struct Line_t {
    char active; // 1=in use, 0=idle

    // as far as known. set e.g. in handleMessage(), when a call comes in.
    unsigned int plci_ncci;

    // the controller this line belongs to. notice that the line is
    // arbitrary, as the B-channels of controllers are usually not ordered.
    int controller;
    int controller_line;

    // the msisdn of the connected or calling instance
    char other_msisdn[20];

    // the msisdn which was called by the other end. Not neccessarily the same as my_msisdn
    char called_msisdn[20];

    // datexp address to be called
    char other_datexp[20];
    
    // my own msisdn. Not neccessarily the same for all lines.
    char*my_msisdn;

    // the job id of the calling job.
    int callingjob;

    int flag_listen; // are we listening for calls?
    int flag_dtmf; // are dtmf enabled/possible with the running connection?
    int flag_fax; // are we connected to a fax or a person?
    int flag_hold; // is there another call waiting? (if yes flag_hold holds it's plci)
    int flag_3pty; // are we connected to 2 partys simultaneously?

    char flag_b3; // true if the line is using a b3 channel 

    state_t state; // the state we are in. see definition of state_t

    unsigned short Appl_Id; // the appl_id of this line. must be used in all send_message calls

    job playjob; // max one per line
    job recordjob; // likewise
    Mutex audio_mutex;
    int supplementaryjob;

    _cmsg*incoming;

    /* these are stored by workerThread */
    Queue* controlMessages;

    /* these variables are a message queue for
     * messages that come in while we are waiting for 
     * something else, e.g. in waitForEvent or in the
     * workerThread.
     */
    Queue* delayedMessages;
    Queue* tmp;

    Thread thread;
    int thread_status; // 0=ended, 1=endrequest, 4=paused, 5=pauserequest

    InputHandler*inputHandler;
    OutputHandler*outputHandler;
};

class CapiHandler2  
{
public:
	Logger*log;
	int allcalls_hack;
	int extern_logger;
	Mutex capi_mutex;

	CapiHandler2(Logger*log);
	virtual ~CapiHandler2();
	
	int open (char*msisdn, char*controller);
	void close (int line);
	ISDNEvent listen (int line, int&timeout);

	void acceptCalls (int line);
	int call (int line, char* msisdn, int mode);
	int play (int line, void*mem, int len); 
	int record (int line, void*buf, int bufsize, void (*callback)(void*,int), void*data);
	int getRecordSize (int line);
	//int fax (int line, void*fax, int faxsize);
	void stop (int line, int jobid);
	int answer (int line, ISDNEvent*event);
	void reject (int line, ISDNEvent*event, int code);
	int forward (int line, char*msisdn);
	void hangup (int line);
	char* eventToString (ISDNEvent*event);
	int checkLine(int line);
	int beep(int line, char*digits);
	int hold (int line);
	int retrieve (int line);
	int conference(int line);
	int endconference(int line);
	int ect(int line);
	void installInputHandler(int line, InputHandler*handler);
	void installOutputHandler(int line, OutputHandler*handler);
	
	void workerThread(int line);
private:
	_cmsg getCapiMessage_internal(int line, int&timeout);

	_cmsg getMessage(int line, int& ms);
	void freeMessage(int line, _cmsg* msg);
	void resetLineToIdle(int line);
	int handleMessage(int line, _cmsg mycmsg, ISDNEvent*e);
	void insertIntoQueue(int line, _cmsg mymsg);

	_cmsg waitForEvent(int line,int event, int subevent, int store);
	int waitForACTIVEIND(int line);
	void transferData(int line);
	int openLine(int line, char*msisdn);

	int init();

	int translatereason(int line, int reason);
        void showSupplementaryReason(int line, short int reason);

	void sendCapiMessage(int line, _cmsg* CMSG);

	int maxchannels;

	struct Line_t channeltab[MAXCHANNELS];
	
	Mutex driver_mutex;
};

#endif

==> caiviar-0.3.4.tar.gz: caiviar-0.3.4/src/CapiHandler2.h <==
/* CapiHandler2.h
 *
header for CapiHandler2.cpp

Part of the caiviar package. 

Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#define MAXCHANNELS 128

#ifndef __capihandler2_h__
#define __capihandler2_h__

#define MSGBUFSIZE 64

#include "../config.h"

#if defined(LINUX) || defined(CYGWIN)
# include <stdio.h>
# include <stdlib.h>
# include "Log.h"

#include "capi20.h"

# include "SimpleCapi.h"
#elif defined(WIN32)

# if _MSC_VER > 1000
#  pragma once
# endif

#include <winsock2.h>
#include <stdlib.h>
#include <malloc.h>

#include "Log.h"

#include "capi20.h"

#include <windef.h>
#include <winbase.h>
#include "SimpleCapi.h"
#endif

#include "os.h"

/* these are the states of some finite state automon.
 * The comments are the other states that can be reached from the
 * actual state. The functions in parentheses are the ones which
 * can be called for the state transition in question
 */
enum state_t
{
    state_idle,  // alertreq, alerted(DONT_DO_ALERT=1), calling(call())
    state_calling,  // connected, idle
    state_alertreq, // alerted
    state_alerted, // connected(answer()), idle(answer() failed)
    state_connected, // disconnecting(hangup from other end), idle(hangup())
    state_disconnecting, // idle
    state_holdreq,
    state_retrievereq,
    state_3ptyreq
};


/* at the moment, this job structure is only used for play and
 * record jobs. call is also a job, but doesn't have a structure.
 * The job id for calls is always 0x0300+line
 */
struct job {
    char * data;
    int data_size;
    char * data_pos;
    int bytes_remaining;
    short unsigned int datahandle;
    int jobid;
    void(*callback)(void*,int);
    void*callback_data;
} ;

class Queue
{
    char* queue;
    int readpos;
    int writepos;
    int number;
    int size;
    int nmemb;
    Mutex mutex;
    public:
    Queue(int size, int nmemb);
    ~Queue();
    int put(void*data);
    int get(void*data);
    int flush();
    int num();
};

struct Line_t {
    char active; // 1=in use, 0=idle

    // as far as known. set e.g. in handleMessage(), when a call comes in.
    unsigned int plci_ncci;

    // the controller this line belongs to. notice that the line is
    // arbitrary, as the B-channels of controllers are usually not ordered.
    int controller;
    int controller_line;

    // the msisdn of the connected or calling instance
    char other_msisdn[20];

    // the msisdn which was called by the other end. Not neccessarily the same as my_msisdn
    char called_msisdn[20];

    // datexp address to be called
    char other_datexp[20];
    
    // my own msisdn. Not neccessarily the same for all lines.
    char*my_msisdn;

    // the job id of the calling job.
    int callingjob;

    int flag_listen; // are we listening for calls?
    int flag_dtmf; // are dtmf enabled/possible with the running connection?
    int flag_fax; // are we connected to a fax or a person?
    int flag_hold; // is there another call waiting? (if yes flag_hold holds it's plci)
    int flag_3pty; // are we connected to 2 partys simultaneously?

    char flag_b3; // true if the line is using a b3 channel 

    state_t state; // the state we are in. see definition of state_t

    unsigned short Appl_Id; // the appl_id of this line. must be used in all send_message calls

    job playjob; // max one per line
    job recordjob; // likewise
    Mutex audio_mutex;
    int supplementaryjob;

    unsigned short connect_resp_messagenumber;

    /* these are stored by workerThread */
    Queue* controlMessages;

    /* these variables are a message queue for
     * messages that come in while we are waiting for 
     * something else, e.g. in waitForEvent or in the
     * workerThread.
     */
    Queue* delayedMessages;
    Queue* tmp;

    Thread thread;
    int thread_status; // 0=ended, 1=endrequest, 4=paused, 5=pauserequest

    InputHandler*inputHandler;
    OutputHandler*outputHandler;
};

class CapiHandler2  
{
public:
	Logger*log;
	int allcalls_hack;
	int extern_logger;
	Mutex capi_mutex;

	CapiHandler2(Logger*log);
	virtual ~CapiHandler2();
	
	int open (char*msisdn, char*controller);
	void close (int line);
	ISDNEvent listen (int line, int&timeout);

	void acceptCalls (int line);
	int call (int line, char* msisdn, int mode);
	int play (int line, void*mem, int len); 
	int record (int line, void*buf, int bufsize, void (*callback)(void*,int), void*data);
	int getRecordSize (int line);
	//int fax (int line, void*fax, int faxsize);
	void stop (int line, int jobid);
	int answer (int line, ISDNEvent*event);
	void reject (int line, ISDNEvent*event, int code);
	int forward (int line, char*msisdn);
	void hangup (int line);
	char* eventToString (ISDNEvent*event);
	int checkLine(int line);
	int beep(int line, char*digits);
	int hold (int line);
	int retrieve (int line);
	int conference(int line);
	int endconference(int line);
	int ect(int line);
	void installInputHandler(int line, InputHandler*handler);
	void installOutputHandler(int line, OutputHandler*handler);
	
	void workerThread(int line);
private:
	_cmsg getCapiMessage_internal(int line, int&timeout);

	_cmsg getMessage(int line, int& ms);
	void freeMessage(int line, _cmsg* msg);
	void resetLineToIdle(int line);
	int handleMessage(int line, _cmsg mycmsg, ISDNEvent*e);
	void insertIntoQueue(int line, _cmsg mymsg);

	_cmsg waitForEvent(int line,int event, int subevent, int store);
	int waitForB3ACTIVEIND(int line);
	void transferData(int line);
	int openLine(int line, char*msisdn);

	int init();

	int translatereason(int line, int reason);
        void showSupplementaryReason(int line, short int reason);

	void sendCapiMessage(int line, _cmsg* CMSG);

	int maxchannels;

	struct Line_t channeltab[MAXCHANNELS];
	
	Mutex driver_mutex;
};

#endif
